# HELPER R CODE FOR PREDICTIVENESS METRICS @ http://stats.drugdesign.fr
# 11-09-2017

# PREDICTIVENESS METRICS:
# Global: Total Gain (TG)
# Partial: partial Total Gain (pTG)

# The particularity of these predictiveness metrics is their ability to directly account for score variation
# in the evaluation of screening results, whereas ROC, BEDROC, RIE & enrichment metrics solely focus on
# the ranking of the compounds.

# INPUT:
# Tabulated screening scores and activity tags.

# OUTPUT:
# Predictiveness curves visualization with global and partial metrics.

# CONTACT:
# Dr. Charly Empereur-mot (charly.empereur@gmail.com)
# Pr. Matthieu Montes (matthieu.montes@cnam.fr)

# Genomics, Bioinformatics & Applications (GBA) Lab.
# Conservatoire National des Arts et Metiers (CNAM)
# 292 rue Saint Martin, 75003 Paris

# REFERENCES:
# Predictiveness Curves in Virtual Screening, 2015, J Cheminform, 7:52 @ https://jcheminf.springeropen.com/articles/10.1186/s13321-015-0100-8
# Screening Explorer – An Interactive Tool for the Analysis of Screening Results, 2016, J Chem Inf Model, 56:12, 2281-2286 @ http://pubs.acs.org/doi/abs/10.1021/acs.jcim.6b00283


##############################################
# FIRST DEMO DATA SET - PREDICTIVENESS CURVE:
DATA = "PPAR_GAMMA_DEMO.csv"
METHODS = c("Surflex-dock", "ICM", "Autodock vina")
THRESHOLD = 0.95  # quantile threshold for pTG calculation and zoomed curve visualization
SCORES_COLUMN = 2 # column number to find scores
ACTIVITY_COLUMN = 3 # column number to find activity tags
COL_PERIOD = 3 # number of columns separating each data set in your text file

tab = read.table(DATA, header=TRUE, sep=" ")

# get glm models for predictiveness curves
preds = list()
for (i in seq(0,length(METHODS)-1)) {
    glm_mdl = glm(tab[,i*COL_PERIOD+ACTIVITY_COLUMN] ~ tab[,i*COL_PERIOD+SCORES_COLUMN], family=binomial("logit"))
    preds[[i+1]] = as.vector(glm_mdl$fitted.values)
}
hitrate = sum(tab[,ACTIVITY_COLUMN])/nrow(tab)

# get predictiveness metrics
tgs = c()
ptgs = c()
for (i in 1:length(METHODS)) {
    tgs = c(tgs, round(sum(abs(rev(preds[[i]])-hitrate)*(1/nrow(tab))) / (2*hitrate*(1-hitrate)), 3))
    ptgs = c(ptgs, round(
        sum(abs(rev(preds[[i]])[(nrow(tab)-(nrow(tab)*(1-THRESHOLD))):nrow(tab)+1]-hitrate)*(1/nrow(tab))) /
        ((1-THRESHOLD)*(1-hitrate)), 3))
    print(paste(METHODS[i], "- TG:", tgs[i]))
    print(paste(METHODS[i], " - pTG @", THRESHOLD,": ", ptgs[i]), sep="")
}

# display curves - complete dataset
dev.new(width=14, height=7)
par(mfrow=c(1,2), pty="s")
plot(1, type="n", xaxt="n", yaxt="n", xlab="Data set quantile", ylab="Probability of activity", xlim=c(1,nrow(tab)), ylim=c(0,1), main="[Demo 1] PPAR Gamma\nPredictiveness Curve - Complete data set")
axis(1, at=seq(0,10)*nrow(tab)/10, labels=seq(0,1,0.1))
axis(2, at=seq(0,1,0.1), labels=seq(0,1,0.1))
abline(h=seq(0,1,0.1), v=seq(0,10)*nrow(tab)/10, lty=2, col="grey")
abline(h=hitrate, lwd=2, lty=2, col="darkgrey")
for (i in 1:length(METHODS)) {
    lines(rev(preds[[i]]), ylim=c(0,1), lwd=2, col=i)
}
legend("topleft", c(METHODS, "Hit-rate"), col=c(seq(1:length(METHODS)),"darkgrey"), text.col=c(seq(1:length(METHODS)),"darkgrey"), inset=0.05, lwd=2, bg="white", lty=c(rep(1,length(METHODS)),2))

# display curves - after selected quantile
plot(1, type="n", xaxt="n", yaxt="n", xlab="Data set quantile", ylab="Probability of activity", xlim=c(0,nrow(tab)-round(THRESHOLD*nrow(tab))), ylim=c(0,1), main=paste("[Demo 1] PPAR Gamma\nPredictiveness Curve - Quantile:",THRESHOLD))
axis(1, at=seq(0,10)*(nrow(tab)-round(THRESHOLD*nrow(tab)))/10, labels=seq(THRESHOLD,1,(1-THRESHOLD)/10))
axis(2, at=seq(0,1,0.1), labels=seq(0,1,0.1))
abline(h=seq(0,1,0.1), v=seq(0,10)*(nrow(tab)-round(THRESHOLD*nrow(tab)))/10, lty=2, col="grey")
abline(h=hitrate, lwd=2, lty=2, col="darkgrey")
for (i in 1:length(METHODS)) {
    lines(rev(preds[[i]])[round(THRESHOLD*nrow(tab)):nrow(tab)+1], ylim=c(0,1), lwd=2, col=i)
}
legend("topleft", c(METHODS, "Hit-rate"), col=c(seq(1:length(METHODS)),"darkgrey"), text.col=c(seq(1:length(METHODS)),"darkgrey"), inset=0.05, lwd=2, bg="white", lty=c(rep(1,length(METHODS)),2))


##############################################
# SECOND DEMO DATA SET - PREDICTIVENESS CURVE:
DATA = "GART_DEMO.csv"
METHODS = c("Surflex-dock", "ICM", "Autodock vina")
THRESHOLD = 0.95  # quantile threshold for pTG calculation and zoomed curve visualization
SCORES_COLUMN = 1 # column number to find scores
ACTIVITY_COLUMN = 2 # column number to find activity tags
COL_PERIOD = 2 # number of columns separating each data set in your text file

tab = read.table(DATA, header=FALSE, sep=" ")
colnames(tab) = c("surf_scores", "surf_activity", "icm_scores", "icm_activity", "vina_scores", "vina_activity")

# get glm models for predictiveness curves
preds = list()
for (i in seq(0,length(METHODS)-1)) {
    glm_mdl = glm(tab[,i*COL_PERIOD+ACTIVITY_COLUMN] ~ tab[,i*COL_PERIOD+SCORES_COLUMN], family=binomial("logit"))
    preds[[i+1]] = as.vector(glm_mdl$fitted.values)
}
hitrate = sum(tab[,ACTIVITY_COLUMN])/nrow(tab)

# get predictiveness metrics
tgs = c()
ptgs = c()
for (i in 1:length(METHODS)) {
    tgs = c(tgs, round(sum(abs(rev(preds[[i]])-hitrate)*(1/nrow(tab))) / (2*hitrate*(1-hitrate)), 3))
    ptgs = c(ptgs, round(
        sum(abs(rev(preds[[i]])[(nrow(tab)-(nrow(tab)*(1-THRESHOLD))):nrow(tab)+1]-hitrate)*(1/nrow(tab))) /
        ((1-THRESHOLD)*(1-hitrate)), 3))
    print(paste(METHODS[i], "- TG:", tgs[i]))
    print(paste(METHODS[i], " - pTG @", THRESHOLD,": ", ptgs[i]), sep="")
}

# display curves - complete dataset
dev.new(width=14, height=7)
par(mfrow=c(1,2), pty="s")
plot(1, type="n", xaxt="n", yaxt="n", xlab="Data set quantile", ylab="Probability of activity", xlim=c(1,nrow(tab)), ylim=c(0,1), main="[Demo 2] GART\nPredictiveness Curve - Complete data set")
axis(1, at=seq(0,10)*nrow(tab)/10, labels=seq(0,1,0.1))
axis(2, at=seq(0,1,0.1), labels=seq(0,1,0.1))
abline(h=seq(0,1,0.1), v=seq(0,10)*nrow(tab)/10, lty=2, col="grey")
abline(h=hitrate, lwd=2, lty=2, col="darkgrey")
for (i in 1:length(METHODS)) {
    lines(rev(preds[[i]]), ylim=c(0,1), lwd=2, col=i)
}
legend("topleft", c(METHODS, "Hit-rate"), col=c(seq(1:length(METHODS)),"darkgrey"), text.col=c(seq(1:length(METHODS)),"darkgrey"), inset=0.05, lwd=2, bg="white", lty=c(rep(1,length(METHODS)),2))

# display curves - after selected quantile
plot(1, type="n", xaxt="n", yaxt="n", xlab="Data set quantile", ylab="Probability of activity", xlim=c(0,nrow(tab)-round(THRESHOLD*nrow(tab))), ylim=c(0,1), main=paste("[Demo 2] GART\nPredictiveness Curve - Quantile:",THRESHOLD))
axis(1, at=seq(0,10)*(nrow(tab)-round(THRESHOLD*nrow(tab)))/10, labels=seq(THRESHOLD,1,(1-THRESHOLD)/10))
axis(2, at=seq(0,1,0.1), labels=seq(0,1,0.1))
abline(h=seq(0,1,0.1), v=seq(0,10)*(nrow(tab)-round(THRESHOLD*nrow(tab)))/10, lty=2, col="grey")
abline(h=hitrate, lwd=2, lty=2, col="darkgrey")
for (i in 1:length(METHODS)) {
    lines(rev(preds[[i]])[round(THRESHOLD*nrow(tab)):nrow(tab)+1], ylim=c(0,1), lwd=2, col=i)
}
legend("topleft", c(METHODS, "Hit-rate"), col=c(seq(1:length(METHODS)),"darkgrey"), text.col=c(seq(1:length(METHODS)),"darkgrey"), inset=0.05, lwd=2, bg="white", lty=c(rep(1,length(METHODS)),2))






